/*---------------------------------------------------------------------------*/
/* HTML-Extractor															 */
/*---------------------------------------------------------------------------*/
/* Programm zum extrahieren von Teilen aus HTML-Seiten						 */
/*---------------------------------------------------------------------------*/
/* Version																	 */
/* 1.0 14.02.2002 Axel Mller												 */
/*---------------------------------------------------------------------------*/

// includes

#include "stdafx.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>

#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/utime.h>

// constants
#define MAX_TOKEN_LEN	100

#define TM_COMMENT 		1
#define TM_NORMAL		2
#define IM_INSERTTAG	1
#define IM_SKIPTAG		2

#define BEFORE_TOKEN		1
#define BETWEEN_TOKEN		2

// variables
char LineBuffer[1000];
char LineBuffer2[1000];
char LineBuffer3[1000];


// functions

int FileExists (char * filename)
{
	FILE *f;
	f = fopen (filename, "r+");
	if (f == NULL)
		return (0);
	fclose (f);
	return (1);
}

int ReadLine (FILE *file, char *line_buf)
{
	char ch;
	int pos;

	printf (".");
	pos = 0;
	ch = fgetc (file);
	if (ch == EOF) return (0);
	while ((ch != EOF) && (ch != 10))
	{
		line_buf[pos++] = ch;
		ch = fgetc (file);
	}
	if (ch == 10) line_buf[pos++] = 10;
	line_buf[pos] = '\0';
	return (1);
}

void BuildTokens (char *text, char *start, char *stop, int tagMode)
{
	if (tagMode == TM_COMMENT)
	{
		strcpy (start, "<!--");
		strcat (start, text);
		strcat (start, "-->");
		strcpy (stop, "<!--/");
		strcat (stop, text);
		strcat (stop, "-->");
	}
	else if (tagMode == TM_NORMAL)
	{
		strcpy (start, "<");
		strcat (start, text);
		strcat (start, ">");
		strcpy (stop, "</");
		strcat (stop, text);
		strcat (stop, ">");
	}
}

char* stristr (const char* source, const char* tag)
{
	int len;
	char* sourceBuffer;
	char* tagBuffer;
	char* pos;

	len = strlen (source);
	sourceBuffer = (char*) malloc (len+1);
	len = strlen (tag);
	tagBuffer = (char*) malloc (len+1);

	strcpy (sourceBuffer, source);
	strupr (sourceBuffer);
	strcpy (tagBuffer, tag);
	strupr (tagBuffer);

	pos = strstr (sourceBuffer, tagBuffer);
	if (pos != NULL)
	{
		pos = ((char*)source) + (pos - sourceBuffer);
	}

	free (tagBuffer);
	free (sourceBuffer);
	return (pos);
}

void ExtractTextBetweenTokens (FILE *sourceFile,
							   char *token,
							   FILE* extractFile,
							   int tagMode,
							   int insertMode)
{
	char StartToken[MAX_TOKEN_LEN];
	char StopToken[MAX_TOKEN_LEN];
	int mode;
	char *start;
	char *poschar;
	char* pos;
	int ok;

	BuildTokens (token, StartToken, StopToken, tagMode);

	mode = BEFORE_TOKEN;
	ok = ReadLine (sourceFile, LineBuffer);
	start = LineBuffer;
	while (ok)
	{
		// printf ("m:%u -^%s^-\n", mode, LineBuffer);
		switch (mode)
		{
			case BEFORE_TOKEN:
				pos = stristr (start, StartToken);
				if (pos == NULL)
				{
					// Token not found
					ok = ReadLine (sourceFile, LineBuffer);
					start = LineBuffer;
				}
				else
				{
					// Token found
					start = pos + strlen(StartToken);
					if (insertMode == IM_INSERTTAG)
					{
						fputs (StartToken, extractFile);
					}
					mode = BETWEEN_TOKEN;
				}
				break;
			case BETWEEN_TOKEN:
				pos = stristr (start, StopToken);
				if (pos == NULL)
				{
					fputs (start, extractFile);
					ok = ReadLine (sourceFile, LineBuffer);
					start = LineBuffer;
				}
				else
				{
					// Bis zum Token schreiben
					poschar = start;
					while (poschar != pos)
					{
						fputc (*poschar, extractFile);
						poschar++;
					}
					start = pos + strlen(StopToken);
					if (insertMode == IM_INSERTTAG)
					{
						fputs (StopToken, extractFile);
					}
					mode = BEFORE_TOKEN;
				}
				break;
		}
	}
}


// main

int main(int argc, char * argv[])
{
	FILE* sourcefile;
	FILE* destfile;
	int insertMode;
	int tagMode;
	char SourceFileName[100];
	char ExtractFileName[100];
	char Tag[MAX_TOKEN_LEN];

	if ((argc <= 2) || (strcmp (argv[1], "/?") == 0))
	{
		printf ("HTMLExtract - Extracts parts of HTML files\n\n");
		printf ("Option 1: HTMLExtract /c <Sourcefile> <Tag> <Extractfile>\n");
		printf ("          Extracts between two comments\n");
		printf ("          <!--Tag--> and <!--/Tag--> into the Extractfile.\n");
		printf ("          The tags itself are not included.\n");
		printf ("Option 2: HTMLExtract /ci <Sourcefile> <Tag> <Extractfile>\n");
		printf ("          Extracts between two comments\n");
		printf ("          <!--Tag--> and <!--/Tag--> into the Extractfile.\n");
		printf ("          The tags itself are included.\n");
		printf ("Option 3: HTMLExtract /n <Sourcefile> <Tag> <Extractfile>\n");
		printf ("          Extracts between two regular HTML tags\n");
		printf ("          <Tag> and </Tag> into the Extractfile.\n");
		printf ("          The tags itself are not included.\n");
		printf ("Option 4: HTMLExtract /ni <Sourcefile> <Tag> <Extractfile>\n");
		printf ("          Extracts between two regular HTML tags\n");
		printf ("          <Tag> and </Tag> into the Extractfile.\n");
		printf ("          The tags itself are included.\n");
		return (1);
	} 
	
	if      (strcmp (argv[1], "/c")  == 0) 
	{
		tagMode		= TM_COMMENT;
		insertMode	= IM_SKIPTAG;
	}
	else if (strcmp (argv[1], "/ci") == 0)
	{
		tagMode		= TM_COMMENT;
		insertMode	= IM_INSERTTAG;
	}
	else if (strcmp (argv[1], "/n") == 0)
	{
		tagMode		= TM_NORMAL;
		insertMode	= IM_SKIPTAG;
	}
	else if (strcmp (argv[1], "/ni") == 0)
	{
		tagMode		= TM_NORMAL;
		insertMode	= IM_INSERTTAG;
	}
	else
	{
		printf ("Please specify a command with '/'.\n\n");
		return (1);
	}

	if (argc != 5)
	{
		printf ("Please give 4 parameters.\n");
		return (1);
	}

	if (!FileExists(argv[2]))
	{
		printf ("HTML file '%s' does not exist!\n\n", argv[2]);
		return (1);
	}


	strcpy (SourceFileName, argv[2]);
	strcpy (Tag, argv[3]);
	strcpy (ExtractFileName, argv[4]);


	sourcefile   = fopen (SourceFileName, "r");
	if (sourcefile == NULL)
	{
		printf ("Couldn't open HTML file '%s'\n", SourceFileName);
		return (1);
	}


	destfile   = fopen (ExtractFileName, "w");
	if (destfile == NULL)
	{
		printf ("Couldn't create extraction file '%s'\n", ExtractFileName);
		fclose (sourcefile);
		return (1);
	}


	ExtractTextBetweenTokens (sourcefile, Tag, destfile, tagMode, insertMode);


	fclose (sourcefile);
	fclose (destfile);

	return 0;
}

